const models = require('../models');
const crypto = require('crypto');
const LdapManager = require('../routes/common/ldap-manager');

class LDAPAuthenticationFailedError extends Error {
  constructor(status, allowedStatuses) {
    super('LDAP could not authenticated provided credentials');
    this.name = 'LDAPAuthenticationFailedError';
  }
}

class LOCALAuthenticationFailedError extends Error {
  constructor(status, allowedStatuses) {
    super('Local Users could not authenticated provided credentials');
    this.name = 'LOCALAuthenticationFailedError';
  }
}

const withLdapAuth = async (req, res, next) => {
    const { username, password } = req.body;
    const pwdHash = crypto
            .createHash('sha256')
            .update(password)
            .digest('hex');
    let localUser = await models.LocalUsers.findOne({
      where: {
        userName: username,
        // password: pwdHash
      }
    });
    if(localUser && localUser.identity_provider == 'Local User') {
      localUser = await models.LocalUsers.findOne({
        where: {
          userName: username,
          password: pwdHash
        }
      });
      if(!localUser) {
        res.locals.setWrongAttempts = {
          error:new LOCALAuthenticationFailedError()
        };
      }
      return next();
    }
    const ldapManager = new LdapManager();
    let isLdapReachable = true;
    var ldapAuth = false;
    models.Ldap.findOne({ where: { } })
      .then(ldap => {
        if (ldap) {
          return ldapManager
            .reuseOrCreateLdapConnection(ldap)
            .then(() => {
              if (ldap.LDAP_user.indexOf('uid=') === 0) {
                let remainingDN = ldap.LDAP_user.split(',');
                remainingDN.shift();
                remainingDN = remainingDN.join(',');
                return ldapManager.bindUser(
                  `uid=${username},${remainingDN}`,
                  password
                );
              } else if(ldap.LDAP_user.split(',')[0] && ldap.LDAP_user.split(',')[0].includes("CN=")){

                let userText = ldap.LDAP_user.toLowerCase();
                userText = userText.substring(userText.indexOf("dc="));
                userText = userText.replace(/dc=/g,'');
                userText = userText.replace(/,/g, '.');

                return ldapManager.bindUser(
                  `${username}@${userText}`,
                  password
                );
              }else {
                return ldapManager.bindUser(
                  `${username}@${ldap.LDAP_user.split('@')[1]}`,
                  password
                );
              }
            })
            .catch(error => {
              if(error.name === 'ADCustomError') {
                return Promise.reject(error)
              }
              if (error.name == 'ConnectionError') {
                isLdapReachable = false;
              }
              return Promise.resolve(false);
            });
        } else {
          return Promise.resolve(false);
        }
      })
      .then(isLdapAuthenticated => {
        ldapAuth = isLdapAuthenticated;
        if (!isLdapAuthenticated) {
          //check in local users
          const hash = crypto
            .createHash('sha256')
            .update(password)
            .digest('hex');
          return models.LocalUsers.findOne({
            where: {
              userName: username,
              password: hash
            }
          });
        } else {
          return Promise.resolve(null);
        }
      })
      .then(localUser => {
        if (isLdapReachable == false && !localUser) {
          console.log('ldap is not reachable');
          const customError = new Error('ldap is not reachable');
          customError.name = 'ADNotReachable';
          throw customError;
        } else if (ldapAuth) {
          return ldapManager.bindDefaultUser();
          //return Promise.resolve();
        } else {
          return localUser
            ? Promise.resolve
            : Promise.reject(new LOCALAuthenticationFailedError());
        }
      })
      .then(() => {
        next();
        return null;
      })
      .catch(err => {
        if(err.name === 'ADCustomError') {
          res.locals.setWrongAttempts = {
            error: err
          };
          return next();
        }
         else if (err.name && err.name == 'ADNotReachable') {
          res.status(503).json({ name: 'ADNotReachable' });
        } else if (err instanceof LOCALAuthenticationFailedError) {
          //res.status(403).json(new LOCALAuthenticationFailedError());
          res.locals.setWrongAttempts = {
            error:new LOCALAuthenticationFailedError()
          };
          next();
        } else {
          res.locals.setWrongAttempts = {
            error:new LDAPAuthenticationFailedError()
          };
          next();
        }
      });
  }

  module.exports = {
      withLdapAuth
  }